<?php

namespace Hrm\Doc\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Hrm\Doc\Http\Requests\DocRequest;
use Hrm\Doc\Interfaces\DocRepositoryInterface;
use Hrm\Doc\Models\Doc;

/**
 * Resource controller class for doc.
 */
class DocResourceController extends BaseController
{

    /**
     * Initialize doc resource controller.
     *
     * @param type DocRepositoryInterface $doc
     *
     * @return null
     */
    public function __construct(DocRepositoryInterface $doc)
    {
        parent::__construct();
        $this->repository = $doc;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Hrm\Doc\Repositories\Criteria\DocResourceCriteria::class);
    }

    /**
     * Display a list of doc.
     *
     * @return Response
     */
    public function index(DocRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Hrm\Doc\Repositories\Presenter\DocPresenter::class)
                ->$function();
        }

        $docs = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('doc::doc.names'))
            ->view('doc::doc.index', true)
            ->data(compact('docs', 'view'))
            ->output();
    }

    /**
     * Display doc.
     *
     * @param Request $request
     * @param Model   $doc
     *
     * @return Response
     */
    public function show(DocRequest $request, Doc $doc)
    {

        if ($doc->exists) {
            $view = 'doc::doc.show';
        } else {
            $view = 'doc::doc.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('doc::doc.name'))
            ->data(compact('doc'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new doc.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DocRequest $request)
    {

        $doc = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('doc::doc.name')) 
            ->view('doc::doc.create', true) 
            ->data(compact('doc'))
            ->output();
    }

    /**
     * Create new doc.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DocRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $doc                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('doc::doc.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('doc/doc/' . $doc->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/doc/doc'))
                ->redirect();
        }

    }

    /**
     * Show doc for editing.
     *
     * @param Request $request
     * @param Model   $doc
     *
     * @return Response
     */
    public function edit(DocRequest $request, Doc $doc)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('doc::doc.name'))
            ->view('doc::doc.edit', true)
            ->data(compact('doc'))
            ->output();
    }

    /**
     * Update the doc.
     *
     * @param Request $request
     * @param Model   $doc
     *
     * @return Response
     */
    public function update(DocRequest $request, Doc $doc)
    {
        try {
            $attributes = $request->all();

            $doc->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('doc::doc.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('doc/doc/' . $doc->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('doc/doc/' . $doc->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the doc.
     *
     * @param Model   $doc
     *
     * @return Response
     */
    public function destroy(DocRequest $request, Doc $doc)
    {
        try {

            $doc->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('doc::doc.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('doc/doc/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('doc/doc/' . $doc->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple doc.
     *
     * @param Model   $doc
     *
     * @return Response
     */
    public function delete(DocRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('doc::doc.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('doc/doc'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/doc/doc'))
                ->redirect();
        }

    }

    /**
     * Restore deleted docs.
     *
     * @param Model   $doc
     *
     * @return Response
     */
    public function restore(DocRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('doc::doc.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/doc/doc'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/doc/doc/'))
                ->redirect();
        }

    }

}
